/*
 * PM - 2016
 * Image Scaler
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define HC595_PORT		PORTB
#define HC595_DDR 		DDRB
#define HC595_DSR_POS	PB3      //Data pin (DS) pin location
#define HC595_DSG_POS	PB4
#define HC595_DSB_POS	PB5
#define HC595_SH_CP_POS	PB1      //Shift Clock (SH_CP) pin location
#define HC595_ST_CP_POS	PB2      //Store Clock (ST_CP) pin location

#define FST_LINE 		PA0
#define SND_LINE 		PA1
#define TRD_LINE		PA2
#define FRT_LINE		PA3
#define FFT_LINE		PA4
#define SXT_LINE		PA5
#define SVT_LINE		PA6
#define LST_LINE		PA7

#define DELAY			3

#define HC595DataHighR()	(HC595_PORT|=(1<<HC595_DSR_POS))
#define HC595DataLowR()		(HC595_PORT&=(~(1<<HC595_DSR_POS)))

#define HC595DataHighG()	(HC595_PORT|=(1<<HC595_DSG_POS))
#define HC595DataLowG()		(HC595_PORT&=(~(1<<HC595_DSG_POS)))

#define HC595DataHighB()	(HC595_PORT|=(1<<HC595_DSB_POS))
#define HC595DataLowB()		(HC595_PORT&=(~(1<<HC595_DSB_POS)))


void init_shift() {
   // Make the Data(DS), Shift clock (SH_CP), Store Clock (ST_CP) lines output
	HC595_DDR|=((1<<HC595_SH_CP_POS)
				|(1<<HC595_ST_CP_POS)
				|(1<<HC595_DSR_POS)
				|(1<<HC595_DSG_POS)
				|(1<<HC595_DSB_POS));
}

void init_line() {
	DDRA = 0xFF;
	PORTA = 0x00;
}

void reset_line(){
	PORTA &= ~((1 << FST_LINE) 
				|(1 << SND_LINE)
				|(1 << TRD_LINE)
				|(1 << FRT_LINE)
				|(1 << FFT_LINE)
				|(1 << SXT_LINE)
				|(1 << SVT_LINE)
				|(1 << LST_LINE));
}


void pulse_shift_clk() {
	HC595_PORT|=(1<<HC595_SH_CP_POS);
	HC595_PORT&=~(1<<HC595_SH_CP_POS);
	//HC595_PORT ^= (1 << HC595_SH_CP_POS);
	//HC595_PORT ^= (1 << HC595_SH_CP_POS);
}

void write(uint8_t data, char color) {
	//Send each 8 bits serially
	//Order is MSB first
	//Latch low
	uint8_t i;
	HC595_PORT&=~(1<<HC595_ST_CP_POS);

	for(i = 0; i < 8; i++) {
		//Output the data on DS line according to the value of MSB
		if(data & 0b10000000) {
			//MSB is 1 so output high
			switch(color) {
				case 'R':
					HC595DataHighR();
					break;
				case 'G':
					HC595DataHighG();
					break;
				case 'B':
					HC595DataHighB();
					break;
				default:
					break;
			}
		}
		else {
			//MSB is 0 so output low
			switch(color) {
				case 'R':
					HC595DataLowR();
					break;
				case 'G':
					HC595DataLowG();
					break;
				case 'B':
					HC595DataLowB();
					break;
				default:
					break;
			}
		}
		pulse_shift_clk();  //Pulse the Clock line
		data = data<<1;  //Now bring next bit at MSB position
	}
	//Now all 8 bits have been transferred to shift register
	//Latch high to light up the pins
	HC595_PORT|=(1<<HC595_ST_CP_POS);
	
}

void scale(void) {
	init_line();
	init_shift();
	while (1) {
		/* Aprind LED-ul. */
	/*	write(0b00000000, 'R', 1);
		write(0b01100110, 'R', 2);
		write(0b11111111, 'R', 3);
		write(0b11111111, 'R', 4);
		write(0b01111110, 'R', 5);
		write(0b00111100, 'R', 6);
		write(0b00011000, 'R', 7);
		write(0b00000000, 'R', 8);
	*/
		
	/*	write(0b00000000, 'R', 1);
		write(0b00000000, 'R', 2);
		write(0b00100100, 'R', 3);
		write(0b01111110, 'R', 4);
		write(0b00111100, 'R', 5);
		write(0b00011000, 'R', 6);
		write(0b00000000, 'R', 7);
		write(0b00000000, 'R', 8);
		*/
		PORTA |= (1 << FST_LINE);
		write(0b00000000, 'G');
		PORTA |= (1 << FST_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << FST_LINE);
		
		write(0b01100110, 'G');
		PORTA |= (1 << SND_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << SND_LINE);
		
		write(0b11111111, 'G');
		PORTA |= (1 << TRD_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << TRD_LINE);
		
		write(0b11111111, 'G');
		PORTA |= (1 << FRT_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << FRT_LINE);
		
		write(0b01111110, 'R');
		PORTA |= (1 << FFT_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << FFT_LINE);
		
		write(0b00111100, 'R');
		PORTA |= (1 << SXT_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << SXT_LINE);
		
		write(0b00011000, 'R');
		PORTA |= (1 << SVT_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << SVT_LINE);
		
		write(0b00000000, 'R');
		PORTA |= (1 << LST_LINE);
		_delay_ms(DELAY);
		PORTA &= ~(1 << LST_LINE);

	}

}

int main(void) {
	scale();

	return 0;
}
